import os
import subprocess

# Define the parameter combinations
configurations = [
    {"PIPELINE": "basic", "DATA_PATH": "/net/travail/bformanek/MRI_dataset", "SPECIALIZATION": "_basic_N1_aug_flip_rot_scale"},
    {"PIPELINE": "basic", "DATA_PATH": "/net/travail/mvajay/TRDP/N2", "SPECIALIZATION": "_basic_N2_aug_flip_rot_scale"},
    {"PIPELINE": "basic", "DATA_PATH": "/net/travail/mvajay/TRDP/N3", "SPECIALIZATION": "_basic_N3_aug_flip_rot_scale"},
    {"PIPELINE": "different_anatomical_plane_in_rgb", "DATA_PATH": "/net/travail/bformanek/MRI_dataset", "SPECIALIZATION": "_different_anatomical_plane_in_rgb_N1_aug_flip_rot_scale"},
    {"PIPELINE": "same_anatomical_plane_in_rgb", "DATA_PATH": "/net/travail/mvajay/TRDP/N3_c1", "SPECIALIZATION": "_same_anatomical_plane_in_rgb_N3_c1_aug_flip_rot_scale"},
    {"PIPELINE": "same_anatomical_plane_in_rgb", "DATA_PATH": "/net/travail/mvajay/TRDP/N3_c2", "SPECIALIZATION": "_same_anatomical_plane_in_rgb_N3_c2_aug_flip_rot_scale"},
    {"PIPELINE": "different_anatomical_plane_ensemble", "DATA_PATH": "/net/travail/bformanek/MRI_dataset", "SPECIALIZATION": "_different_anatomical_plane_ensemble_N1_aug_flip_rot_scale"},
    {"PIPELINE": "ensemble_same_anatomical_planes_in_RGB", "DATA_PATH": "/net/travail/mvajay/TRDP/N3_c1", "SPECIALIZATION": "_ensemble_same_anatomical_planes_in_RGB_N3_c1_aug_flip_rot_scale"},
    {"PIPELINE": "ensemble_same_anatomical_planes_in_RGB", "DATA_PATH": "/net/travail/mvajay/TRDP/N3_c2", "SPECIALIZATION": "_ensemble_same_anatomical_planes_in_RGB_N3_c2_aug_flip_rot_scale"}
]

# Path to parameters.py and main.py
parameters_file = "parameters.py"
main_script = "main.py"

# Backup the original parameters file
if not os.path.exists(f"{parameters_file}.backup"):
    os.rename(parameters_file, f"{parameters_file}.backup")

# Function to update parameters.py
def update_parameters(config):
    with open(parameters_file, "w") as file:
        file.write(f"""PIPELINE = "{config['PIPELINE']}"\n""")
        file.write(f"""DATA_PATH = "{config['DATA_PATH']}"\n""")
        file.write(f"""SPECIALIZATION = "{config['SPECIALIZATION']}"\n\n""")
        file.write("""
# COMMON PARAMETERS
DEVICE = 'cuda' # 'cuda' or 'cpu'

TRAIN_FOLDER = DATA_PATH + '/train'
VAL_FOLDER = DATA_PATH + '/val'
TEST_FOLDER = DATA_PATH + '/test'
INVIDUALS = 1940
                   
MODEL_NAME = 'resnet18'

BATCH_SIZE = 64
WORKERS = 8
EPOCHS = 60

SAVE_FOLDER_MAIN = '/net/travail/mvajay/TRDP/'

CHECKPOINTS_FOLDER = SAVE_FOLDER_MAIN+'checkpoints/transfer_checkpoints_'+ MODEL_NAME + SPECIALIZATION
SAVE_PREDICTION_PATH = SAVE_FOLDER_MAIN+'predictions/' + MODEL_NAME + SPECIALIZATION + "_b_epoch_"
SAVE_MISCLASSIFIED_PATH = SAVE_FOLDER_MAIN+'misclassified_data/'+ MODEL_NAME + SPECIALIZATION +'_misclassified'
SAVE_CONFUSIONM_PATH = SAVE_FOLDER_MAIN + 'confusionM/' + MODEL_NAME + SPECIALIZATION + '_majority_voting'+'.png'
SAVE_INPUT_VISUALIZATION_PATH = SAVE_FOLDER_MAIN+'input_visualization/' + MODEL_NAME + SPECIALIZATION
SAVE_PATH_LEARNING_CURVES = SAVE_FOLDER_MAIN + "learning_curves/" + MODEL_NAME + SPECIALIZATION

LOG_PATH = SAVE_FOLDER_MAIN+'logs/'+ MODEL_NAME + SPECIALIZATION + ".log"

AXIS_CONVERTION = {'0': "axial",'1': "coronial",'2': "sagittal"}
""")

# Iterate over each configuration
for config in configurations:
    print(f"Running configuration: {config}")
    # Update the parameters.py file
    update_parameters(config)
    # Execute the main.py script
    try:
        subprocess.run(["python", main_script], check=True)
    except subprocess.CalledProcessError as e:
        print(f"Error while running configuration {config}: {e}")

# Restore the original parameters.py file
os.rename(f"{parameters_file}.backup", parameters_file)
